import pytest
from mastermind import MasterMind
from mastermind.exceptions import InvalidUserInput


@pytest.fixture()
def game_instance():
    mm = MasterMind()
    return mm


def test_default_player_name(game_instance):
    assert game_instance.player_name == "player"


def test_player_name(game_instance):
    game_instance.set_player_name("John Rambo")
    assert game_instance.player_name == "John Rambo"


def test_new_game_should_not_end(game_instance):
    assert game_instance.end_game == False


def test_round_should_continue_not_matching_guess(game_instance):
    guess = 1
    number = 9
    result = game_instance._round_continue(guess, number)
    assert result == True


def test_round_should_not_continue_matching_guess(game_instance):
    guess = 9
    number = 9
    result = game_instance._round_continue(guess, number)
    assert result == False


def test_user_input_game_ends_with_end_keyword(game_instance):
    result = game_instance._user_ends_game("end")
    assert result == True


@pytest.mark.parametrize('user_input', ["blahend", "", "nd", "e nd", "endfoo"])
def test_user_input_game_continues(game_instance, user_input):
    result = game_instance._user_ends_game(user_input)
    assert result == False


@pytest.mark.parametrize('user_input', ["-1", "0", "11", "99", "93982938792837982739"])
def test_parse_guess_exception_out_of_range(game_instance, user_input):
    with pytest.raises(InvalidUserInput) as error:
        game_instance._parse_guess(user_input)
    assert "Guess is out of range 1 - 10" in str(error.value)


def test_parse_guess_returns_integer(game_instance):
    guess = game_instance._parse_guess("7")
    assert type(guess) is int


@pytest.mark.parametrize('user_input', ["1", "2", "3", "4", "5", "6", "7", "8", "9", "10"])
def test_parse_guess_returns_same_number(game_instance, user_input):
    guess = game_instance._parse_guess(user_input)
    number = int(user_input)
    assert guess == number

